/* ----------------------------------------------------------------------------
 * pbb_errno.c
 * libpbb funtions to handle error messages
 *
 * Copyright 2002 Matthias Grimm
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 * ----------------------------------------------------------------------------*/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <unistd.h>
#include <syslog.h>
#include <stdarg.h>
#include <string.h>

#if defined(HAVE_ICONV) && defined(HAVE_LANGIINFO_CODESET) && defined(ENABLE_NLS)
#  include <iconv.h>
#  include <langinfo.h>
#  include <libintl.h>
#endif

#include "pbb.h"

extern struct libbase libdata;

void
print_error(char *msg, ...)
{
	struct libbase *base = &libdata;
	va_list list;
	char inbuffer[200], *inptr = inbuffer;

#if defined(HAVE_ICONV) && defined(HAVE_LANGIINFO_CODESET) && defined(ENABLE_NLS)
	iconv_t cvh;
	char outbuffer[200], *outptr = outbuffer;
	size_t count, inidx, outidx;
#endif
	
	va_start (list, msg);
	vsnprintf (inbuffer, sizeof(inbuffer), msg, list);
	va_end (list);
	
#if defined(HAVE_ICONV) && defined(HAVE_LANGIINFO_CODESET) && defined(ENABLE_NLS)
	inidx = strlen(inbuffer);
	outidx = sizeof(outbuffer);
	
	if ((cvh = iconv_open (nl_langinfo(CODESET),
				bind_textdomain_codeset(textdomain(NULL), NULL))) != (iconv_t) -1) {
		count = iconv (cvh, &inptr, &inidx, &outptr, &outidx);
		inptr = count == -1 ? inbuffer : outbuffer;
		*outptr = '\0';  /* iconv doesn't terminate strings with '\0' */
		iconv_close (cvh);
	}
#endif
	
	if (base->daemon != PBBDS_NONE) {
		syslog(LOG_ERR, inptr);
	} else
		fprintf(stderr, inptr);
}

